Package: openPrimeR
Title: Multiplex PCR Primer Design and Analysis
Version: 1.33.0
Authors@R: c(person("Matthias", "Döring", 
                    email = "matthias-doering@gmx.de", 
                    role = c("aut", "cre")),
            person("Nico", "Pfeifer", 
                    email = "pfeifer@informatik.uni-tuebingen.de", 
                    role = c("aut"))
            )
Description: An implementation of methods for designing, evaluating,
    and comparing primer sets for multiplex PCR.
    Primers are designed by solving a set cover problem such that
    the number of covered template sequences is maximized with
    the smallest possible set of primers.
    To guarantee that high-quality primers are generated,
    only primers fulfilling constraints on their physicochemical properties 
    are selected. A Shiny app providing a user interface for the functionalities
    of this package is provided by the 'openPrimeRui' package.
Depends: R (>= 4.0.0)
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: Biostrings (>= 2.38.4), pwalign, XML (>= 3.98-1.4), scales (>=
        0.4.0), reshape2 (>= 1.4.1), seqinr (>= 3.3-3), IRanges (>=
        2.4.8), GenomicRanges (>= 1.22.4), ggplot2 (>= 2.1.0), plyr (>=
        1.8.4), dplyr (>= 0.5.0), stringdist (>= 0.9.4.1), stringr (>=
        1.0.0), RColorBrewer (>= 1.1-2), DECIPHER (>= 1.16.1),
        lpSolveAPI (>= 5.5.2.0-17), digest (>= 0.6.9), Hmisc (>=
        3.17-4), ape (>= 3.5), BiocGenerics (>= 0.16.1), S4Vectors (>=
        0.8.11), foreach (>= 1.4.3), magrittr (>= 1.5), uniqtag (>=
        1.0), openxlsx (>= 4.0.17), grid (>= 3.1.0), grDevices (>=
        3.1.0), stats (>= 3.1.0), utils (>= 3.1.0), methods (>= 3.1.0)
Suggests: testthat (>= 1.0.2), knitr (>= 1.13), rmarkdown (>= 1.0),
        devtools (>= 1.12.0), doParallel (>= 1.0.10), pander (>=
        0.6.0), learnr (>= 0.9)
SystemRequirements: MAFFT (>= 7.305), OligoArrayAux (>= 3.8), ViennaRNA
        (>= 2.4.1), MELTING (>= 5.1.1), Pandoc (>= 1.12.3)
biocViews: Software, Technology, Coverage, MultipleComparison
VignetteBuilder: knitr
Collate: 'AnalysisStats.R' 'Comparison.R' 'Data.R' 'templates.R'
        'primers.R' 'IO.R' 'IO_view.R' 'Input.R' 'Ippolito.R'
        'Output.R' 'Plots.R' 'PrimerDesign.R' 'PrimerEvaluation.R'
        'RefCoverage.R' 'Scoring.R' 'SettingsDoc.R' 'TemplatesDoc.R'
        'Tiller.R' 'ambiguity.R' 'check_stop_codons.R'
        'con_annealing_temperature.R' 'con_dimerization.R'
        'con_gc_clamp.R' 'con_gc_ratio.R' 'con_melting_temperature.R'
        'con_primer_coverage.R' 'con_primer_efficiency.R'
        'con_primer_secondary_structures.R' 'con_repeats.R'
        'con_runs.R' 'con_template_secondary_structures.R'
        'constraints.R' 'constraints_eval.R' 'errors.R' 'filters.R'
        'helper_functions.R' 'initialize_primers.R'
        'initialize_primers_tree.R' 'openPrimeR.R' 'optimization_ILP.R'
        'optimization_algo.R' 'optimization_global.R'
        'optimization_greedy.R' 'plots_comparison.R' 'settings.R'
        'plots_constraints.R' 'plots_coverage.R' 'plots_filtering.R'
        'primer_significance.R' 'startApp.R' 'zzz.R'
git_url: https://git.bioconductor.org/packages/openPrimeR
git_branch: devel
git_last_commit: 9ac7cf4
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 23:16:08 UTC; biocbuild
Author: Matthias Döring [aut, cre],
  Nico Pfeifer [aut]
Maintainer: Matthias Döring <matthias-doering@gmx.de>
