# Generated by roxygen2: do not edit by hand

S3method(assign_binding_regions,character)
S3method(assign_binding_regions,numeric)
S3method(cbind,Primers)
S3method(cbind,Templates)
S3method(cbind,fill)
S3method(rbind,Primers)
S3method(rbind,Templates)
export(ConstraintOptions)
export(ConstraintSettings)
export(CoverageConstraints)
export(PCR_Conditions)
export(Primers)
export(Templates)
export(adjust_binding_regions)
export(assign_binding_regions)
export(check_constraints)
export(check_restriction_sites)
export(classify_design_problem)
export(create_coverage_xls)
export(create_report)
export(design_primers)
export(filter_primers)
export(get_comparison_table)
export(get_cvg_ratio)
export(get_cvg_stats)
export(get_cvg_stats_primer)
export(get_initial_primers)
export(parallel_setup)
export(plot_conservation)
export(plot_constraint)
export(plot_constraint_deviation)
export(plot_constraint_fulfillment)
export(plot_cvg_constraints)
export(plot_cvg_vs_set_size)
export(plot_penalty_vs_set_size)
export(plot_primer)
export(plot_primer_binding_regions)
export(plot_primer_cvg)
export(plot_primer_subsets)
export(plot_template_cvg)
export(primer_significance)
export(read_primers)
export(read_settings)
export(read_templates)
export(runTutorial)
export(score_conservation)
export(score_degen)
export(score_primers)
export(select_regions_by_conservation)
export(subset_primer_set)
export(update_template_cvg)
export(write_primers)
export(write_settings)
export(write_templates)
exportClasses(ConstraintOptions)
exportClasses(ConstraintSettings)
exportClasses(CoverageConstraints)
exportClasses(DesignSettings)
exportClasses(PCR_Conditions)
exportClasses(Primers)
exportClasses(Templates)
exportMethods("$<-")
exportMethods("PCR<-")
exportMethods("[")
exportMethods("conOptions<-")
exportMethods("constraintLimits<-")
exportMethods("constraints<-")
exportMethods("cvg_constraints<-")
exportMethods(PCR)
exportMethods(cbind2)
exportMethods(conOptions)
exportMethods(constraintLimits)
exportMethods(constraints)
exportMethods(cvg_constraints)
exportMethods(rbind2)
import(ggplot2)
import(lpSolveAPI)
import(methods)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,unlist)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,DNAStringSetList)
importFrom(Biostrings,DNA_BASES)
importFrom(Biostrings,IUPAC_CODE_MAP)
importFrom(Biostrings,extractAt)
importFrom(Biostrings,matchPattern)
importFrom(Biostrings,mergeIUPACLetters)
importFrom(Biostrings,reverseComplement)
importFrom(Biostrings,vmatchPattern)
importFrom(Biostrings,width)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IRanges)
importFrom(IRanges,Views)
importFrom(IRanges,as.matrix)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,overlapsAny)
importFrom(RColorBrewer,brewer.pal)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(foreach,getDoParRegistered)
importFrom(foreach,getDoParWorkers)
importFrom(grDevices,colorRampPalette)
importFrom(magrittr,"%>%")
importFrom(plyr,.)
importFrom(plyr,arrange)
importFrom(plyr,catcolwise)
importFrom(plyr,ddply)
importFrom(plyr,here)
importFrom(plyr,numcolwise)
importFrom(plyr,rbind.fill)
importFrom(plyr,summarize)
importFrom(pwalign,compareStrings)
importFrom(pwalign,mismatch)
importFrom(pwalign,nucleotideSubstitutionMatrix)
importFrom(pwalign,pairwiseAlignment)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(stats,as.formula)
importFrom(stats,ave)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,sd)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,setTxtProgressBar)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.csv)
importFrom(utils,write.table)
