#' @rdname Data
#' @name Data
#'
#' @usage data(Tiller)
#'
#' @aliases Tiller tiller.template.df tiller.primer.df tiller.settings
#'
#' @references Tiller, Thomas, et al. "Efficient generation of monoclonal antibodies from single human B cells by single cell RT-PCR and expression vector cloning." Journal of immunological methods 329.1 (2008): 112-124.
#' @examples
#'
#' # Load the data from Tiller et al.
#' data(Tiller)
#' tiller.primer.df
#' tiller.template.df
#' constraints(tiller.settings)
NULL
