#' @rdname Data
#' @name Data
#'
#' @usage data(Ippolito)
#'
#' @aliases template.df primer.df settings
#'
#' @references Ippolito GC, Hoi KH, Reddy ST, Carroll SM, Ge X, Rogosch T, 
#' Zemlin M, Shultz LD, Ellington AD, VanDenBerg CL, Georgiou G. 2012. 
#' Antibody Repertoires in Humanized NOD-scid-IL2R gamma null Mice and Human B
#' Cells Reveals Human-Like Diversification and Tolerance Checkpoints 
#' in the Mouse. PLoS One 7:e35497.
#' @examples
#' 
#' # Load the data from Ippolito et al.
#' data(Ippolito)
#' primer.df
#' template.df
#' constraints(settings)
NULL
