% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.R
\name{listToXml}
\alias{listToXml}
\title{List to XML}
\usage{
listToXml(item, tag)
}
\arguments{
\item{item}{}

\item{tag}{xml tag}
}
\value{
xmlNode
}
\description{
Convert List to XML.
}
\details{
Can convert list or other object to an xml object using xmlNode.
}
\author{
David LeBauer, Carl Davidson, Rob Kooper
}
\keyword{internal}
